library verilog;
use verilog.vl_types.all;
entity axi_fifo is
    generic(
        DATA_WIDTH      : integer := 32;
        ADDR_WIDTH      : integer := 32;
        ID_WIDTH        : integer := 8;
        AWUSER_ENABLE   : integer := 0;
        AWUSER_WIDTH    : integer := 1;
        WUSER_ENABLE    : integer := 0;
        WUSER_WIDTH     : integer := 1;
        BUSER_ENABLE    : integer := 0;
        BUSER_WIDTH     : integer := 1;
        ARUSER_ENABLE   : integer := 0;
        ARUSER_WIDTH    : integer := 1;
        RUSER_ENABLE    : integer := 0;
        RUSER_WIDTH     : integer := 1;
        WRITE_FIFO_DEPTH: integer := 32;
        READ_FIFO_DEPTH : integer := 32;
        WRITE_FIFO_DELAY: integer := 0;
        READ_FIFO_DELAY : integer := 0
    );
    port(
        clk             : in     vl_logic;
        rst             : in     vl_logic;
        s_axi_awid      : in     vl_logic_vector;
        s_axi_awaddr    : in     vl_logic_vector;
        s_axi_awlen     : in     vl_logic_vector(7 downto 0);
        s_axi_awsize    : in     vl_logic_vector(2 downto 0);
        s_axi_awburst   : in     vl_logic_vector(1 downto 0);
        s_axi_awlock    : in     vl_logic;
        s_axi_awcache   : in     vl_logic_vector(3 downto 0);
        s_axi_awprot    : in     vl_logic_vector(2 downto 0);
        s_axi_awqos     : in     vl_logic_vector(3 downto 0);
        s_axi_awregion  : in     vl_logic_vector(3 downto 0);
        s_axi_awuser    : in     vl_logic_vector;
        s_axi_awvalid   : in     vl_logic;
        s_axi_awready   : out    vl_logic;
        s_axi_wdata     : in     vl_logic_vector;
        s_axi_wstrb     : in     vl_logic_vector;
        s_axi_wlast     : in     vl_logic;
        s_axi_wuser     : in     vl_logic_vector;
        s_axi_wvalid    : in     vl_logic;
        s_axi_wready    : out    vl_logic;
        s_axi_bid       : out    vl_logic_vector;
        s_axi_bresp     : out    vl_logic_vector(1 downto 0);
        s_axi_buser     : out    vl_logic_vector;
        s_axi_bvalid    : out    vl_logic;
        s_axi_bready    : in     vl_logic;
        s_axi_arid      : in     vl_logic_vector;
        s_axi_araddr    : in     vl_logic_vector;
        s_axi_arlen     : in     vl_logic_vector(7 downto 0);
        s_axi_arsize    : in     vl_logic_vector(2 downto 0);
        s_axi_arburst   : in     vl_logic_vector(1 downto 0);
        s_axi_arlock    : in     vl_logic;
        s_axi_arcache   : in     vl_logic_vector(3 downto 0);
        s_axi_arprot    : in     vl_logic_vector(2 downto 0);
        s_axi_arqos     : in     vl_logic_vector(3 downto 0);
        s_axi_arregion  : in     vl_logic_vector(3 downto 0);
        s_axi_aruser    : in     vl_logic_vector;
        s_axi_arvalid   : in     vl_logic;
        s_axi_arready   : out    vl_logic;
        s_axi_rid       : out    vl_logic_vector;
        s_axi_rdata     : out    vl_logic_vector;
        s_axi_rresp     : out    vl_logic_vector(1 downto 0);
        s_axi_rlast     : out    vl_logic;
        s_axi_ruser     : out    vl_logic_vector;
        s_axi_rvalid    : out    vl_logic;
        s_axi_rready    : in     vl_logic;
        m_axi_awid      : out    vl_logic_vector;
        m_axi_awaddr    : out    vl_logic_vector;
        m_axi_awlen     : out    vl_logic_vector(7 downto 0);
        m_axi_awsize    : out    vl_logic_vector(2 downto 0);
        m_axi_awburst   : out    vl_logic_vector(1 downto 0);
        m_axi_awlock    : out    vl_logic;
        m_axi_awcache   : out    vl_logic_vector(3 downto 0);
        m_axi_awprot    : out    vl_logic_vector(2 downto 0);
        m_axi_awqos     : out    vl_logic_vector(3 downto 0);
        m_axi_awregion  : out    vl_logic_vector(3 downto 0);
        m_axi_awuser    : out    vl_logic_vector;
        m_axi_awvalid   : out    vl_logic;
        m_axi_awready   : in     vl_logic;
        m_axi_wdata     : out    vl_logic_vector;
        m_axi_wstrb     : out    vl_logic_vector;
        m_axi_wlast     : out    vl_logic;
        m_axi_wuser     : out    vl_logic_vector;
        m_axi_wvalid    : out    vl_logic;
        m_axi_wready    : in     vl_logic;
        m_axi_bid       : in     vl_logic_vector;
        m_axi_bresp     : in     vl_logic_vector(1 downto 0);
        m_axi_buser     : in     vl_logic_vector;
        m_axi_bvalid    : in     vl_logic;
        m_axi_bready    : out    vl_logic;
        m_axi_arid      : out    vl_logic_vector;
        m_axi_araddr    : out    vl_logic_vector;
        m_axi_arlen     : out    vl_logic_vector(7 downto 0);
        m_axi_arsize    : out    vl_logic_vector(2 downto 0);
        m_axi_arburst   : out    vl_logic_vector(1 downto 0);
        m_axi_arlock    : out    vl_logic;
        m_axi_arcache   : out    vl_logic_vector(3 downto 0);
        m_axi_arprot    : out    vl_logic_vector(2 downto 0);
        m_axi_arqos     : out    vl_logic_vector(3 downto 0);
        m_axi_arregion  : out    vl_logic_vector(3 downto 0);
        m_axi_aruser    : out    vl_logic_vector;
        m_axi_arvalid   : out    vl_logic;
        m_axi_arready   : in     vl_logic;
        m_axi_rid       : in     vl_logic_vector;
        m_axi_rdata     : in     vl_logic_vector;
        m_axi_rresp     : in     vl_logic_vector(1 downto 0);
        m_axi_rlast     : in     vl_logic;
        m_axi_ruser     : in     vl_logic_vector;
        m_axi_rvalid    : in     vl_logic;
        m_axi_rready    : out    vl_logic
    );
end axi_fifo;
